import 'docker.dart';
import 'image.dart';
import 'images.dart';

class Container {
  String containerid;
  String imageid;
  String created;
  String status;
  String ports;
  String name;

  Container({
    required this.containerid,
    required this.imageid,
    required this.created,
    required this.status,
    required this.ports,
    required this.name,
  });

  bool isSame(Container other) => containerid == other.containerid;

  @override
  bool operator ==(covariant Container other) {
    if (this == other) return true;

    if (containerid == other.containerid) return true;

    return false;
  }

  Image? get image => Images().findByImageId(imageid);

  void stop() {
    if (isRunning) {
      dockerRun('stop', containerid);
    }
  }

  void start({List<String>? args, String? argString, bool daemon = true}) {
    var cmdArgs = containerid;

    if (args != null) {
      cmdArgs += ' ' + args.join(' ');
    }
    if (argString != null) {
      cmdArgs += ' ' + argString;
    }

    if (!daemon) {
      cmdArgs = '--attach --interactive $cmdArgs';
    }
    dockerRun('start', cmdArgs);
  }

  bool get isRunning {
    return dockerRun(
                'container', "inspect -f '{{.State.Running}}' $containerid")
            .first ==
        'true';
  }

  /// deletes this docker container.
  void delete() {
    dockerRun('container', 'rm $containerid');
  }

  /// writes this containers docker logs to the console
  /// If [limit] is 0 (the default) all log lines a output.
  /// If [limit] is > 0 then only the last [limit] lines are output.
  void showLogs({int limit = 0}) {
    var limitFlag = '';
    if (limit != 0) {
      limitFlag = '-n $limit';
    }
    dockerRun('logs', '$limitFlag $containerid');
  }

  /// Attaches to the running container and starts a bash command prompt.
  void cli() {
    dockerRun('exec', '-it $containerid /bin/bash');
  }

  @override
  String toString() {
    return '$containerid ${image?.fullname} $status $name';
  }
}
