import 'package:dcli/dcli.dart';
import 'package:docker_cli/src/image.dart';
import 'container.dart';
import 'images.dart';

class Containers {
  factory Containers() => _self;
  Containers._internal();

  static final _self = Containers._internal();

  ////final containerCache = <Container>[];

  List<Container> containers({bool excludeStopped = false}) {
    final containerCache = <Container>[];

    //if (containerCache.isEmpty) {
    var cmd =
        'docker container ls --format "table {{.ID}}|{{.Image}}|{{.CreatedAt}}|{{.Status}}|{{.Ports}}|{{.Names}}"';
    if (!excludeStopped) cmd += ' --all';
    final lines = cmd.toList(skipLines: 1);

    for (final line in lines) {
      final parts = line.split('|');
      final containerid = parts[0];
      var imageid = parts[1];
      final created = parts[2];
      final status = parts[3];
      final ports = parts[4];
      final name = parts[5];

      // sometimes the imageid is actually the image name.
      final image = Images().findByFullname(imageid);
      if (image != null) {
        /// the imageid that we parsed actually contained an image name
        /// so lets replace that with the actual id.
        imageid = image.imageid!;
      }

      final container = Container(
          containerid: containerid,
          imageid: imageid,
          created: created,
          status: status,
          ports: ports,
          name: name);
      containerCache.add(container);
      //}
    }
    return containerCache;
  }

  // void flushCache() {
  //   containerCache.clear();
  // }

  bool existsByContainerId(String containerid, {bool excludeStopped = false}) =>
      findByContainerId(containerid, excludeStopped: excludeStopped) != null;

  bool existsByName({required String name, bool excludeStopped = false}) =>
      findByName(name, excludeStopped: excludeStopped) != null;

  Container? findByContainerId(String containerid,
      {bool excludeStopped = false}) {
    final list = containers(excludeStopped: excludeStopped);

    if (containerid.length > 12) {
      containerid = containerid.substring(0, 12);
    }

    for (final container in list) {
      if (containerid == container.containerid) {
        return container;
      }
    }
    return null;
  }

  List<Container> findByImageid(String? imageid,
      {bool excludeStopped = false}) {
    final list = containers(excludeStopped: excludeStopped);
    final matches = <Container>[];

    for (final container in list) {
      if (imageid == container.imageid) {
        matches.add(container);
      }
    }
    return matches;
  }

  /// if [excludeStopped] is true then exclude containers that are not running.
  Container? findByName(String name, {bool excludeStopped = false}) {
    final list = containers(excludeStopped: excludeStopped);

    for (final container in list) {
      if (name == container.name) {
        return container;
      }
    }
    return null;
  }

  List<Container> findByImage(Image image, {bool excludeStopped = false}) =>
      findByImageid(image.imageid, excludeStopped: excludeStopped);
}
