import 'package:dcli/dcli.dart';

import 'image.dart';

class ImageName {
  String? registry;
  String? repository;
  late String name;
  String? tag;

  ImageName(this.registry, this.repository, this.name, this.tag);

  ImageName.fromName(String fullname) {
    final parsed = _parseName(fullname);
    name = parsed.name;
    repository = parsed.repository;
    registry = parsed.registry;
    tag = parsed.tag;

    // return ImageName(_fullname.registry, _fullname.repository, _fullname.name,
    //     _fullname.tag);
  }

  static ImageName fromRepositoryAndName(String repositoryAndName,
      {String? tag}) {
    var _repositoryAndName = repositoryAndName;

    if (tag != null) _repositoryAndName += ':$tag';

    final parsed = _parseName(_repositoryAndName);

    return ImageName(
        parsed.registry, parsed.repository, parsed.name, parsed.tag);
  }

  String get fullname {
    var fname = '';
    if (repository != null) fname += '$repository\\';

    fname += name;

    if (tag != null) fname += ':$tag';
    return fname;
  }

  /// Returns true if the passed [image] has the same
  /// name as this image.
  bool isSameFullname(Image image) {
    return (repository == image.repository &&
        name == image.name &&
        tag == image.tag);
  }

  /// Takes a docker repo/name:tag string and splits it into
  /// three components.
  /// The repo and tag are optional.
  /// If no tag is provided then 'latest' is assumed.
  /// e.g.
  /// repo/name:tag
  /// repo/name
  /// name
  /// name:tag
  static ImageName _parseName(String fullname) {
    String? registry;
    String? repository;
    String name;
    String? tag;

    Settings().verbose(fullname);

    if (fullname.contains('/')) {
      var parts = fullname.split('/');
      if (parts.length == 3) {
        // we have registry/repo/name
        registry = parts[0];
        parts.removeAt(0);
      }
      var repoOrRegistry = parts[0];
      // If the part contains a dot then it must be
      // a domain name which is only valid for the
      // registry.
      if (repoOrRegistry.contains('.')) {
        // We should only see this if the no.
        // of parts is true as the user passed
        // registry/name
        assert(registry == null);
        registry = repoOrRegistry;
        // repository = null;
      } else {
        repository = repoOrRegistry;
        // registry = null;
      }
      if (parts[1].contains(':')) {
        parts = parts[1].split(':');
        name = parts[0];
        tag = parts[1];
      } else {
        name = parts[1];
      }
    } else {
      if (fullname.contains(':')) {
        final parts = fullname.split(':');
        name = parts[0];
        tag = parts[1];
      } else {
        name = fullname;
      }
    }

    if (tag == null || tag.isEmpty) tag = 'latest';

    if (name.isEmpty) {
      throw ArgumentError(
          'The name component may not be empty. Invalid fullname: $fullname');
    }
    return ImageName(registry, repository, name, tag);
  }
}
