import 'package:dcli/dcli.dart';
import 'docker.dart';
import 'image.dart';

class Images {
  factory Images() => _self;
  Images._internal();

  static final _self = Images._internal();

  // final imageCache = <Image>[];

  List<Image> get images {
    final imageCache = <Image>[];
    //if (imageCache.isEmpty) {
    var lines = 'docker images'.toList(skipLines: 1);

    const cmd =
        'docker images --format "table {{.ID}}|{{.Repository}}|{{.Tag}}|{{.CreatedAt}}|{{.Size}}"';
    // print(cmd);

    lines = cmd.toList(skipLines: 1);

    for (final line in lines) {
      final parts = line.split('|');
      final imageid = parts[0];
      final repositoryAndName = parts[1];
      final tag = parts[2];
      final created = parts[3];
      final size = parts[4];

      final image = Image(
          repositoryAndName: repositoryAndName,
          tag: tag,
          imageid: imageid,
          created: created,
          size: size);
      imageCache.add(image);
    }
    // }

    return imageCache;
  }

  // void flushCache() {
  //   imageCache.clear();
  // }

  bool existsByImageId({required String imageid}) =>
      findByImageId(imageid) != null;

  bool existsByFullname({
    required String fullname,
  }) =>
      findByFullname(fullname) != null;

  bool existsByParts(
          {required String repository,
          required String name,
          required String tag}) =>
      findByParts(repository: repository, name: name, tag: tag).isNotEmpty;

  Image? findByImageId(String imageid) {
    final list = images;

    for (final image in list) {
      if (imageid == image.imageid) {
        return image;
      }
    }
    return null;
  }

  /// full name of the image using the format repo/name:tag
  /// The repo component is optional but the name and tag
  /// must be passed otherwise an [ArgumentError]
  /// will be thrown.
  Image? findByFullname(String fullname) {
    final match = Image.fromName(fullname);

    if (match.tag == null) {
      throw ArgumentError('You must provide the name and tag, found $fullname');
    }
    Settings().verbose('Match ${match.repository} ${match.name} ${match.tag}');

    var list = findByParts(
        repository: match.repository, name: match.name, tag: match.tag);
    if (list.length > 1) throw StateError(fullname);
    if (list.isEmpty) return null;
    return list[0];
  }

  /// Returns a list of images that match the passed
  /// parts. The [name] part is the only compulsory part.
  /// If no matches are found then an empty list is returned
  List<Image> findByParts(
      {required String? repository,
      required String name,
      required String? tag}) {
    final list = images;
    var found = <Image>[];

    for (final image in list) {
      if (image.isSame(repository: repository, name: name, tag: tag)) {
        found.add(image);
      }
    }

    return found;
  }

  Image? pull({required String fullname}) {
    dockerRun('pull', fullname);
    // flushCache();
    return findByFullname(fullname);
  }
}
